#!/bin/perl

#----------------------------------------------------------
# txt2dxf
#
# Reading translated TEXT/MTEXT/DIMENSION/ATTDEF blocks from TXT file
# and updating corresponding TEXT/MTEXT/DIMENSION/ATTDER blocks in DXF file.
#----------------------------------------------------------
# Method:
#   1. Find TEXT/MTEXT/DIMENSION/ATTDEF header in DXF
#   2. Skipping header and attribute lines
#   3. Read text from TXT file and substitute it
#      for original text.
#----------------------------------------------------------
# Command line parameters:
#   -----------------------------------
#   Parameters        | Descriptions
#   -----------------------------------
#   dxf-file name(s)  | Name(s) of DXF file to process
#   (wildcards enabled)
#
# Input files:
#   1. DXF-file(s)
#
#   TXT file(s) with TEXT/MTEXT/DIMENSION blocks. Format:
#
#      {1-nnn}
#      <Text line from MTEXT block>
#      {2-nnn}
#      <Text line from TEXT block>
#      {3-nnn}
#      <Text line from DIMENSION block>
#      {4-nnn}
#      <Text line from ATTDEF block>
#
#   nnn is the ordinal number of TEXT or MTEXT block.
#   (TEXT and MTEXT block sequences are numbered separately).
#
# Output files:
#   DXF-file with replaced text blocks. Naming convention:
#   <file>.DXF - original file.
#   <file>_TR.DXF - translated file.
#----------------------------------------------------------
# (C) Copyright 2006, Mikegor.
#
# Version |    Date     | Comment
#---------+-------------+--------------------
#   1.0   | 16 Nov 2006 | Original code.
#   1.1   | 25 Dec 2006 | Processing DIMENSION blocks.
#   2.0   | 26 Mar 2008 | Multiple files processing enabled.
#   2.1   | 09 Apr 2008 | Processing of ATTDEF blocks.
#----------------------------------------------------------

#---------------------------------------------------
#--- 0. Initialization
#---------------------------------------------------

#--- 0.1. Printing programm version

print "Import of translated text into DXF files. Version 2.0, (c) Mikegor, 26 Mar 2008.\n";


#--- 0.2. Getting number of arguments

$narg = $#ARGV + 1;


#--- 0.3. Prining help message if no arguments

if ($narg == 0)
{
print "Usage:\n";
print "txt2dxf file1[.dxf] file2[.dxf] ...\n";
exit
}


#--- 0.4. Loop over arguments

for ($iarg=0; $iarg <= $#ARGV; $iarg++)
{


#---------------------------------------------------
#--- 1. Opening files
#---------------------------------------------------

#--- 1.1. Input dxf file

$dxf = $ARGV[$iarg];

if (! ($dxf =~ /\.dxf$/))
{
   $dxf .= ".dxf";
}

print "Opening $dxf\n";

open (DXF, '<', $dxf) or die "Cannot open '$dxf'\n";


#--- 1.2. Output txt file

$txt = $ARGV[$iarg];

if ($txt =~ /\.dxf$/)
{
   $txt =~ s/\.dxf$/.txt/;
}
else
{
   $txt .= ".txt";
}

print "Opening $txt\n";
open (TXT, '<', $txt) or die "Cannot open '$txt'\n";

#--- 1.3. Output dxf file

$dxft = $dxf;

if ($dxft =~ /\.dxf$/)
{
   $dxft =~ s/\.dxf$/_TR.dxf/;
}
else
{
   $dxft .= "_TR.dxf";
}

print "Opening $dxft\n";
open (DXFT, '>', $dxft) or die "Cannot open '$dxft'\n";


#---------------------------------------------------
#--- 2. Parsing file
#---------------------------------------------------

#--- 2.0. Initialization

$i       = 0;
$jmtxt   = 0;
$jtxt    = 0;
$jdim    = 0;
$jatt    = 0;

#--- 2.1. Reading file

while ($s = <DXF>)
{
   
   #--- 2.1.1. Special character processing
   
   ++$i;
   $s =~ s/[\r\n]*$//g;
   print DXFT "$s\n";

   #--- 2.1.2. Processing MTEXT block

   if ($s =~ /^ *AcDbMText *$/)
   {

      #--- 2.1.2.1. Skipping attribute lines

      for($k = 1; $k <= 14; $k++)
      {
         $s = <DXF>;
         $s =~ s/[\r\n]*$//g;
         print DXFT "$s\n";
         ++$i;
      }

      #--- 2.1.2.2. Collecting text

      do
      {
         $id = <DXF>;
         $id =~ s/[\r\n]*$//g;
         ++$i;
         if (($id != 3) && ($id != 1))
         {
            die "Stop here: something is wrong in line $i\n";
         }
         $txtline .= <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
      } until $id == 1;

      #--- 2.1.2.3. Writing text

      ++$jmtxt;
      $txtline = <TXT>;
      $txtline = <TXT>;
      $txtline =~ s/[\r\n]*$//g;
      print DXFT "$id\n";
      print DXFT "$txtline\n";
   }
   

   #--- 2.1.3. Processing TEXT block

   if ($s =~ /^ *AcDbText *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      print DXFT "$s\n";
      ++$i;
      
      if ($s == 10)
      {
         for($k = 1; $k <= 8; $k++)
         {
            $s = <DXF>;
            $s =~ s/[\r\n]*$//g;
            print DXFT "$s\n";
            ++$i;
         }
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
         if ($txtline !~ /^ *$/)
         {
            ++$jtxt;
            $txtline = <TXT>;
            $txtline = <TXT>;
            $txtline =~ s/[\r\n]*$//g;
         }
         print DXFT "$txtline\n";
      }

   }

   #--- 2.1.4. Processing DIMENSION block

   if ($s =~ /^ *AcDbDimension *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      print DXFT "$s\n";
      ++$i;
      
      if ($s == 2)
      {
         for($k = 1; $k <= 16; $k++)
         {
            $s = <DXF>;
            $s =~ s/[\r\n]*$//g;
            print DXFT "$s\n";
            ++$i;
         }
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;
         if ($txtline !~ /^ *$/)
         {
            ++$jdim;
            $txtline = <TXT>;
            $txtline = <TXT>;
            $txtline =~ s/[\r\n]*$//g;
         }
         print DXFT "$txtline\n";
      }

   }

   #--- 2.1.5. Processing ATTDEF block

   if ($s =~ /^ *AcDbAttributeDefinition *$/)
   {
      $s = <DXF>;
      $s =~ s/[\r\n]*$//g;
      print DXFT "$s\n";
      ++$i;
      
      if ($s == 3)
      {
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;

         ++$jatt;
         $txtline = <TXT>;
         $txtline = <TXT>;
         $txtline =~ s/[\r\n]*$//g;
         print DXFT "$txtline\n";
         $txtline = "";

         $s = <DXF>;
         $s =~ s/[\r\n]*$//g;
         print DXFT "$s\n";
         ++$i;
         $txtline = <DXF>;
         $txtline =~ s/[\r\n]*$//g;
         ++$i;

         $txtline = <TXT>;
         $txtline = <TXT>;
         $txtline =~ s/[\r\n]*$//g;
         print DXFT "$txtline\n";
         $txtline = "";
      }

   }

}

print "Statistics:\n";
print "$i lines parsed\n";
print "$jmtxt MTEXT blocks imported\n";
print "$jtxt TEXT blocks imported\n";
print "$jdim DIMENSION blocks imported\n";
print "$jatt ATTDEF blocks imported\n";

#---------------------------------------------------
#--- 3. Closing files
#---------------------------------------------------

close DXF;
close TXT;


#---------------------------------------------------
#--- 4. Finalization
#---------------------------------------------------

#--- 4.1. Closing main loop

}
